package controller

import (
	"fmt"
	"strconv"
	"time"

	"be.ems/src/framework/constants"
	"be.ems/src/framework/i18n"
	"be.ems/src/framework/reqctx"
	"be.ems/src/framework/resp"
	"be.ems/src/framework/utils/date"
	"be.ems/src/framework/utils/file"
	"be.ems/src/framework/utils/parse"
	"be.ems/src/modules/system/model"
	"be.ems/src/modules/system/service"

	"github.com/gin-gonic/gin"
)

// 实例化控制层 SysRoleController 结构体
var NewSysRole = &SysRoleController{
	sysRoleService: service.NewSysRole,
	sysUserService: service.NewSysUser,
}

// 角色信息
//
// PATH /system/role
type SysRoleController struct {
	sysRoleService *service.SysRole // 角色服务
	sysUserService *service.SysUser // 用户服务
}

// 角色列表
//
// GET /list
//
//	@Tags			system/role
//	@Accept			json
//	@Produce		json
//	@Param			roleName	query		string	false	"roleName"
//	@Param			pageNum		query		number	true	"pageNum"	default(1)
//	@Param			pageSize	query		number	true	"pageSize"	default(10)
//	@Success		200			{object}	object	"Response Results"
//	@Security		TokenAuth
//	@Summary		Role Information List
//	@Description	Role Information List
//	@Router			/system/role/list [get]
func (s *SysRoleController) List(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	query := reqctx.QueryMap(c)
	// 多语言值转key查询
	if v, ok := query["roleName"]; ok && v != "" {
		query["roleName"] = i18n.TFindKeyPrefix(language, "role", v)
	}
	rows, total := s.sysRoleService.FindByPage(query)

	// 闭包函数处理多语言
	converI18n := func(language string, arr *[]model.SysRole) {
		for i := range *arr {
			(*arr)[i].RoleName = i18n.TKey(language, (*arr)[i].RoleName)
			(*arr)[i].Remark = i18n.TKey(language, (*arr)[i].Remark)
		}
	}
	converI18n(language, &rows)

	c.JSON(200, resp.OkData(map[string]any{"rows": rows, "total": total}))
}

// 角色信息详情
//
// GET /:roleId
func (s *SysRoleController) Info(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	roleId := parse.Number(c.Param("roleId"))
	if roleId <= 0 {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: roleId is empty"))
		return
	}
	data := s.sysRoleService.FindById(roleId)
	if data.RoleId == roleId {
		data.RoleName = i18n.TKey(language, data.RoleName)
		data.Remark = i18n.TKey(language, data.Remark)
		c.JSON(200, resp.OkData(data))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// 角色信息新增
//
// POST /
func (s *SysRoleController) Add(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	var body model.SysRole
	if err := c.ShouldBindBodyWithJSON(&body); err != nil {
		errMsgs := fmt.Sprintf("bind err: %s", resp.FormatBindError(err))
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_PARSER, errMsgs))
		return
	}
	if body.RoleId > 0 {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: roleId not is empty"))
		return
	}

	// 判断角色名称是否唯一
	uniqueName := s.sysRoleService.CheckUniqueByName(body.RoleName, 0)
	if !uniqueName {
		// msg := fmt.Sprintf("角色新增【%s】失败，角色名称已存在", body.RoleName)
		msg := i18n.TTemplate(language, "role.errNameExists", map[string]any{"name": body.RoleName})
		c.JSON(200, resp.ErrMsg(msg))
		return
	}

	// 判断角色键值是否唯一
	uniqueKey := s.sysRoleService.CheckUniqueByKey(body.RoleKey, 0)
	if !uniqueKey {
		//  msg := fmt.Sprintf("角色新增【%s】失败，角色键值已存在", body.RoleName)
		msg := i18n.TTemplate(language, "role.errKeyExists", map[string]any{"name": body.RoleName})
		c.JSON(200, resp.ErrMsg(msg))
		return
	}

	body.CreateBy = reqctx.LoginUserToUserName(c)
	insertId := s.sysRoleService.Insert(body)
	if insertId > 0 {
		c.JSON(200, resp.OkData(insertId))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// 角色信息修改
//
// PUT /
func (s *SysRoleController) Edit(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	var body model.SysRole
	if err := c.ShouldBindBodyWithJSON(&body); err != nil {
		errMsgs := fmt.Sprintf("bind err: %s", resp.FormatBindError(err))
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_PARSER, errMsgs))
		return
	}
	if body.RoleId <= 0 {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: roleId is empty"))
		return
	}

	// 检查是否系统管理员角色
	if body.RoleId == constants.SYS_ROLE_SYSTEM_ID {
		// c.JSON(200, resp.ErrMsg("不允许操作系统管理员角色"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "app.common.errOperateRole")))
		return
	}

	// 检查是否存在
	roleInfo := s.sysRoleService.FindById(body.RoleId)
	if roleInfo.RoleId != body.RoleId {
		// c.JSON(200, resp.ErrMsg("没有权限访问角色数据！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.noData")))
		return
	}

	// 判断角色名称是否唯一
	uniqueName := s.sysRoleService.CheckUniqueByName(body.RoleName, body.RoleId)
	if !uniqueName {
		//  msg := fmt.Sprintf("角色修改【%s】失败，角色名称已存在", body.RoleName)
		msg := i18n.TTemplate(language, "role.errNameExists", map[string]any{"name": body.RoleName})
		c.JSON(200, resp.ErrMsg(msg))
		return
	}

	// 判断角色键值是否唯一
	uniqueKey := s.sysRoleService.CheckUniqueByKey(body.RoleKey, body.RoleId)
	if !uniqueKey {
		//  msg := fmt.Sprintf("角色修改【%s】失败，角色键值已存在", body.RoleName)
		msg := i18n.TTemplate(language, "role.errKeyExists", map[string]any{"name": body.RoleName})
		c.JSON(200, resp.ErrMsg(msg))
		return
	}

	// 多语言非原始值
	i18nValue := i18n.TKey(language, roleInfo.RoleName)
	if i18nValue != roleInfo.RoleName {
		service.NewSysI18n.UpdateKeyValue(language, roleInfo.RoleName, body.RoleName)
		body.RoleName = roleInfo.RoleName
	}
	// 多语言非原始值
	i18nValue2 := i18n.TKey(language, roleInfo.Remark)
	if i18nValue2 != roleInfo.Remark {
		service.NewSysI18n.UpdateKeyValue(language, roleInfo.Remark, body.Remark)
		body.Remark = roleInfo.Remark
	}

	roleInfo.Remark = body.Remark
	roleInfo.RoleName = body.RoleName
	roleInfo.RoleKey = body.RoleKey
	roleInfo.RoleSort = body.RoleSort
	roleInfo.DataScope = body.DataScope
	roleInfo.MenuCheckStrictly = body.MenuCheckStrictly
	roleInfo.DeptCheckStrictly = body.DeptCheckStrictly
	roleInfo.StatusFlag = body.StatusFlag
	roleInfo.UpdateBy = reqctx.LoginUserToUserName(c)
	roleInfo.MenuIds = body.MenuIds
	roleInfo.DeptIds = body.DeptIds
	rows := s.sysRoleService.Update(roleInfo)
	if rows > 0 {
		c.JSON(200, resp.Ok(nil))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// Remove 角色信息删除
//
// DELETE /:roleId
func (s *SysRoleController) Remove(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	roleId := c.Param("roleId")
	if roleId == "" {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: roleId is empty"))
		return
	}

	// 处理字符转id数组后去重
	uniqueIDs := parse.RemoveDuplicatesToArray(roleId, ",")
	// 转换成int64数组类型
	ids := make([]int64, 0)
	for _, v := range uniqueIDs {
		ids = append(ids, parse.Number(v))
	}

	// 检查是否系统管理员角色
	for _, id := range ids {
		if id == constants.SYS_ROLE_SYSTEM_ID {
			// c.JSON(200, resp.ErrMsg("不允许操作系统管理员角色"))
			c.JSON(200, resp.ErrMsg(i18n.TKey(language, "app.common.errOperateRole")))
			return
		}
	}

	rows, err := s.sysRoleService.DeleteByIds(ids)
	if err != nil {
		c.JSON(200, resp.ErrMsg(err.Error()))
		return
	}
	// msg := fmt.Sprintf("删除成功：%d", rows)
	msg := i18n.TTemplate(language, "app.common.deleteSuccess", map[string]any{"num": rows})
	c.JSON(200, resp.OkMsg(msg))
}

// Status 角色状态变更
//
// PUT /status
func (s SysRoleController) Status(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	var body struct {
		RoleID     int64  `json:"roleId" binding:"required"`               // 角色ID
		StatusFlag string `json:"statusFlag" binding:"required,oneof=0 1"` // 状态
	}
	if err := c.ShouldBindBodyWithJSON(&body); err != nil {
		errMsgs := fmt.Sprintf("bind err: %s", resp.FormatBindError(err))
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_PARSER, errMsgs))
		return
	}

	// 检查是否系统管理员角色
	if body.RoleID == constants.SYS_ROLE_SYSTEM_ID {
		// c.JSON(200, resp.ErrMsg("不允许操作系统管理员角色"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "app.common.errOperateRole")))
		return
	}

	// 检查是否存在
	role := s.sysRoleService.FindById(body.RoleID)
	if role.RoleId != body.RoleID {
		// c.JSON(200, resp.ErrMsg("没有权限访问角色数据！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.noData")))
		return
	}

	// 与旧值相等不变更
	if role.StatusFlag == body.StatusFlag {
		// c.JSON(200, resp.ErrMsg("变更状态与旧值相等！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.statusEq")))
		return
	}

	// 更新状态不刷新缓存
	role.StatusFlag = body.StatusFlag
	role.UpdateBy = reqctx.LoginUserToUserName(c)
	rows := s.sysRoleService.Update(role)
	if rows > 0 {
		c.JSON(200, resp.Ok(nil))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// DataScope 角色数据权限修改
//
// PUT /data-scope
func (s *SysRoleController) DataScope(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	var body struct {
		RoleId            int64   `json:"roleId" binding:"required"`                      // 角色ID
		DeptIds           []int64 `json:"deptIds"`                                        // 部门组（数据权限）
		DataScope         string  `json:"dataScope" binding:"required,oneof=1 2 3 4 5"`   // 数据范围（1：全部数据权限 2：自定数据权限 3：本部门数据权限 4：本部门及以下数据权限 5：仅本人数据权限）
		DeptCheckStrictly string  `json:"deptCheckStrictly" binding:"required,oneof=0 1"` // 部门树选择项是否关联显示（0：父子不互相关联显示 1：父子互相关联显示）
	}
	if err := c.ShouldBindBodyWithJSON(&body); err != nil {
		errMsgs := fmt.Sprintf("bind err: %s", resp.FormatBindError(err))
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_PARSER, errMsgs))
		return
	}

	// 检查是否系统管理员角色
	if body.RoleId == constants.SYS_ROLE_SYSTEM_ID {
		// c.JSON(200, resp.ErrMsg("不允许操作系统管理员角色"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "app.common.errOperateRole")))
		return
	}

	// 检查是否存在
	roleInfo := s.sysRoleService.FindById(body.RoleId)
	if roleInfo.RoleId != body.RoleId {
		// c.JSON(200, resp.ErrMsg("没有权限访问角色数据！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.noData")))
		return
	}

	// 更新数据权限
	roleInfo.DeptIds = body.DeptIds
	roleInfo.DataScope = body.DataScope
	roleInfo.DeptCheckStrictly = body.DeptCheckStrictly
	roleInfo.UpdateBy = reqctx.LoginUserToUserName(c)
	rows := s.sysRoleService.UpdateAndDataScope(roleInfo)
	if rows > 0 {
		c.JSON(200, resp.Ok(nil))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// UserAuthList 角色分配用户列表
//
// GET /user/list
func (s SysRoleController) UserAuthList(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	roleId := parse.Number(c.Query("roleId"))
	if roleId <= 0 {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: roleId is empty"))
		return
	}

	// 检查是否存在
	role := s.sysRoleService.FindById(roleId)
	if role.RoleId != roleId {
		// c.JSON(200, resp.ErrMsg("没有权限访问角色数据！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.noData")))
		return
	}

	query := reqctx.QueryMap(c)
	dataScopeSQL := reqctx.LoginUserToDataScopeSQL(c, "sys_user", "sys_user")
	rows, total := s.sysUserService.FindAuthUsersPage(query, dataScopeSQL)
	c.JSON(200, resp.OkData(map[string]any{"rows": rows, "total": total}))
}

// UserAuthChecked 角色分配选择授权
//
// PUT /user/auth
func (s SysRoleController) UserAuthChecked(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	var body struct {
		RoleId  int64   `json:"roleId" binding:"required"`  // 角色ID
		UserIds []int64 `json:"userIds" binding:"required"` // 用户ID组
		Auth    bool    `json:"auth"`                       // 选择操作 添加true 取消false
	}
	if err := c.ShouldBindBodyWithJSON(&body); err != nil {
		errMsgs := fmt.Sprintf("bind err: %s", resp.FormatBindError(err))
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_PARSER, errMsgs))
		return
	}
	if len(body.UserIds) <= 0 {
		c.JSON(422, resp.CodeMsg(resp.CODE_PARAM_CHEACK, "bind err: userIds is empty"))
		return
	}

	// 检查是否存在
	role := s.sysRoleService.FindById(body.RoleId)
	if role.RoleId != body.RoleId {
		// c.JSON(200, resp.ErrMsg("没有权限访问角色数据！"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "role.noData")))
		return
	}

	var rows int64
	if body.Auth {
		rows = s.sysRoleService.InsertAuthUsers(body.RoleId, body.UserIds)
	} else {
		rows = s.sysRoleService.DeleteAuthUsers(body.RoleId, body.UserIds)
	}
	if rows > 0 {
		c.JSON(200, resp.Ok(nil))
		return
	}
	c.JSON(200, resp.Err(nil))
}

// 导出角色信息
//
// POST /export
func (s *SysRoleController) Export(c *gin.Context) {
	language := reqctx.AcceptLanguage(c)
	// 查询结果，根据查询条件结果，单页最大值限制
	query := reqctx.QueryMap(c)
	rows, total := s.sysRoleService.FindByPage(query)
	if total == 0 {
		// c.JSON(200, resp.CodeMsg(40016, "export data record as empty"))
		c.JSON(200, resp.ErrMsg(i18n.TKey(language, "app.common.exportEmpty")))
		return
	}

	// 闭包函数处理多语言
	converI18n := func(language string, arr *[]model.SysRole) {
		for i := range *arr {
			(*arr)[i].RoleName = i18n.TKey(language, (*arr)[i].RoleName)
			(*arr)[i].Remark = i18n.TKey(language, (*arr)[i].Remark)
		}
	}
	converI18n(language, &rows)

	// 导出文件名称
	fileName := fmt.Sprintf("role_export_%d_%d.xlsx", len(rows), time.Now().UnixMilli())
	// 第一行表头标题
	headerCells := map[string]string{
		"A1": i18n.TKey(language, "role.export.id"),
		"B1": i18n.TKey(language, "role.export.name"),
		"C1": i18n.TKey(language, "role.export.key"),
		"D1": i18n.TKey(language, "role.export.sort"),
		"E1": i18n.TKey(language, "role.export.status"),
		"F1": i18n.TKey(language, "role.export.time"),
		// "F1": i18n.TKey(language, "role.export.dataScope"),
	}
	// 读取系统角色数据范围字典数据
	// dictSysRoleDatascope := s.sysDictDataService.SelectDictDataByType("sys_role_datascope")
	// 从第二行开始的数据
	dataCells := make([]map[string]any, 0)
	for i, row := range rows {
		idx := strconv.Itoa(i + 2)
		// 数据范围
		// dataScope := row.DataScope
		// for _, v := range dictSysRoleDatascope {
		// 	if row.DataScope == v.DictValue {
		// 		dataScope = i18n.TKey(language, v.DictLabel)
		// 		break
		// 	}
		// }
		// 角色状态
		statusValue := i18n.TKey(language, "dictData.disable")
		if row.StatusFlag == "1" {
			statusValue = i18n.TKey(language, "dictData.normal")
		}
		dataCells = append(dataCells, map[string]any{
			"A" + idx: row.RoleId,
			"B" + idx: row.RoleName,
			"C" + idx: row.RoleKey,
			"D" + idx: row.RoleSort,
			"E" + idx: statusValue,
			"F" + idx: date.ParseDateToStr(row.CreateTime, date.YYYY_MM_DDTHH_MM_SSZ),
			// "F" + idx: dataScope,
		})
	}

	// 导出数据表格
	saveFilePath, err := file.WriteSheet(headerCells, dataCells, fileName, "")
	if err != nil {
		c.JSON(200, resp.ErrMsg(err.Error()))
		return
	}

	c.FileAttachment(saveFilePath, fileName)
}
